@@ -16,30 +16,67 @@ include!("no_atomic.rs");
 const LATEST_STABLE: Version =
     Version { minor: 59, nightly: false, commit_date: Date::new(2022, 2, 23) };
 
+// Probe if unstable features can be compiled with the expected API.
+// This prevents accidental depends on them if the upstream changes its API.
+// This is the same approach used in autocfg, anyhow, etc.
+// for aarch64 and x86_64 macos
 const PROBE_ATOMIC_128: &str = r#"
 #![no_std]
-#![allow(stable_features)]
 #![feature(integer_atomics)]
 fn _probe() {
-    let v = core::sync::atomic::AtomicU128::new(0);
-    let _ = v.swap(1, core::sync::atomic::Ordering::Relaxed);
+    let v = core::sync::atomic::AtomicU128::new(0_u128);
+    let _: u128 = v.swap(1_u128, core::sync::atomic::Ordering::Relaxed);
 }
 "#;
+// for x86_64
 const PROBE_CMPXCHG16B: &str = r#"
 #![no_std]
-#![allow(stable_features)]
 #![feature(stdsimd, cmpxchg16b_target_feature)]
 #[allow(unused_unsafe)]
 #[target_feature(enable = "cmpxchg16b")]
-unsafe fn _probe(dst: *mut u128) -> u128 {
+unsafe fn _probe(dst: *mut u128) {
     unsafe {
-        core::arch::x86_64::cmpxchg16b(
+        let _: u128 = core::arch::x86_64::cmpxchg16b(
             dst,
             0_u128,
             0_u128,
             core::sync::atomic::Ordering::Relaxed,
             core::sync::atomic::Ordering::Relaxed,
-        )
+        );
+    }
+}
+"#;
+// for s390x
+const PROBE_ATOMIC_INTRINSICS: &str = r#"
+#![no_std]
+#![feature(core_intrinsics)]
+#[allow(unused_unsafe)]
+unsafe fn _probe(dst: *mut u128) {
+    unsafe {
+        let _: u128 = core::intrinsics::atomic_load_acq(dst);
+        let _: u128 = core::intrinsics::atomic_load_relaxed(dst);
+        let _: u128 = core::intrinsics::atomic_load(dst);
+        let _: () = core::intrinsics::atomic_store_rel(dst, 0_u128);
+        let _: () = core::intrinsics::atomic_store_relaxed(dst, 0_u128);
+        let _: () = core::intrinsics::atomic_store(dst, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_acq(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_rel(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_acqrel(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_relaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_acq_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_acqrel_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchg_failacq(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_acq(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_rel(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_acqrel(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_relaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_acq_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_acqrel_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_failrelaxed(dst, 0_u128, 0_u128);
+        let _: (u128, bool) = core::intrinsics::atomic_cxchgweak_failacq(dst, 0_u128, 0_u128);
     }
 }
 "#;
@@ -58,14 +95,18 @@ fn main() {
             return;
         }
     };
-    // HACK: If --target is specified, rustflags is not applied to the build
-    // script itself, so the build script will not be rerun when these are changed.
-    println!("cargo:rerun-if-env-changed=RUSTFLAGS");
-    println!("cargo:rerun-if-env-changed=CARGO_BUILD_RUSTFLAGS");
-    println!(
-        "cargo:rerun-if-env-changed=CARGO_TARGET_{}_RUSTFLAGS",
-        target.to_uppercase().replace('-', "_").replace('.', "_")
-    );
+    let aarch64 = target.starts_with("aarch64");
+    let x86_64 = target.starts_with("x86_64");
+    if aarch64 || x86_64 {
+        // HACK: If --target is specified, rustflags is not applied to the build
+        // script itself, so the build script will not be rerun when these are changed.
+        println!("cargo:rerun-if-env-changed=RUSTFLAGS");
+        println!("cargo:rerun-if-env-changed=CARGO_BUILD_RUSTFLAGS");
+        println!(
+            "cargo:rerun-if-env-changed=CARGO_TARGET_{}_RUSTFLAGS",
+            target.to_uppercase().replace('-', "_").replace('.', "_")
+        );
+    }
 
     let version = match rustc_version() {
         Some(version) => version,
@@ -122,16 +163,16 @@ fn main() {
     }
 
     if target.starts_with("thumbv6m-") && target.contains("-none") {
-        println!("cargo:rustc-cfg=thumbv6m");
+        println!("cargo:rustc-cfg=portable_atomic_armv6m");
     }
     // #[cfg(not(target_feature = "v6"))] doesn't work on stable.
     if target.starts_with("armv5te-") {
-        println!("cargo:rustc-cfg=armv5te");
+        println!("cargo:rustc-cfg=portable_atomic_armv5te");
     }
 
     // aarch64_target_feature stabilized in Rust 1.61.
     // aarch64 macos always support lse: https://github.com/rust-lang/rust/blob/1.59.0/compiler/rustc_target/src/spec/aarch64_apple_darwin.rs#L5
-    if target.starts_with("aarch64")
+    if aarch64
         && (version.minor >= 59 || version.nightly)
         && has_target_feature("lse", target == "aarch64-apple-darwin", &version, Some(61))
     {
@@ -139,8 +180,7 @@ fn main() {
     }
 
     // cmpxchg16b is available via asm (1.59+) or stdsimd (nightly).
-    let may_use_cmpxchg16b =
-        target.starts_with("x86_64") && (version.minor >= 59 || version.nightly);
+    let may_use_cmpxchg16b = x86_64 && (version.minor >= 59 || version.nightly);
     let mut has_cmpxchg16b = false;
     if may_use_cmpxchg16b {
         // x86_64 macos always support cmpxchg16b: https://github.com/rust-lang/rust/blob/1.59.0/compiler/rustc_target/src/spec/x86_64_apple_darwin.rs#L7
@@ -155,21 +195,30 @@ fn main() {
         println!("cargo:rustc-cfg=portable_atomic_nightly");
 
         // `cfg(sanitize = "..")` is not stabilized.
-        let tsan = std::env::var("CARGO_CFG_SANITIZE").unwrap_or_default().contains("thread");
-        if tsan {
+        let sanitize = std::env::var("CARGO_CFG_SANITIZE").unwrap_or_default();
+        if sanitize.contains("thread") {
             println!("cargo:rustc-cfg=sanitize_thread");
         }
 
-        if HAS_ATOMIC_128.contains(&&*target) && probe(PROBE_ATOMIC_128, &target).unwrap_or(false) {
-            println!("cargo:rustc-cfg=portable_atomic_core_atomic_128");
-        } else if may_use_cmpxchg16b
-            && (has_cmpxchg16b || cfg!(feature = "fallback") && cfg!(feature = "outline-atomics"))
-            && probe(PROBE_CMPXCHG16B, &target).unwrap_or(false)
-        {
-            println!("cargo:rustc-cfg=portable_atomic_cmpxchg16b_stdsimd");
-            if cfg!(feature = "fallback") && cfg!(feature = "outline-atomics") {
-                println!("cargo:rustc-cfg=portable_atomic_cmpxchg16b_dynamic");
+        if aarch64 || x86_64 {
+            if HAS_ATOMIC_128.contains(&&*target)
+                && probe(PROBE_ATOMIC_128, &target).unwrap_or(false)
+            {
+                println!("cargo:rustc-cfg=portable_atomic_core_atomic_128");
+            } else if may_use_cmpxchg16b
+                && (has_cmpxchg16b
+                    || cfg!(feature = "fallback") && cfg!(feature = "outline-atomics"))
+                && probe(PROBE_CMPXCHG16B, &target).unwrap_or(false)
+            {
+                println!("cargo:rustc-cfg=portable_atomic_cmpxchg16b_stdsimd");
+                if cfg!(feature = "fallback") && cfg!(feature = "outline-atomics") {
+                    println!("cargo:rustc-cfg=portable_atomic_cmpxchg16b_dynamic");
+                }
             }
+        } else if target.starts_with("s390x")
+            && probe(PROBE_ATOMIC_INTRINSICS, &target).unwrap_or(false)
+        {
+            println!("cargo:rustc-cfg=portable_atomic_s390x_atomic_128");
         }
     }
 }
@@ -255,7 +304,8 @@ fn probe(code: &str, target: &str) -> Option<bool> {
         .arg("--crate-type=lib")
         .arg("--out-dir")
         .arg(out_dir)
-        .arg("--emit=llvm-ir");
+        .arg("--emit=llvm-ir")
+        .arg("--cap-lints=warn");
 
     cmd.arg("--target").arg(target);
 
