@@ -1,6 +1,6 @@
 // Atomic{I,U}128 implementation for x86_64 using cmpxchg16b (DWCAS).
 
-#[path = "cmpxchg16b_detect.rs"]
+#[path = "detect/cmpxchg16b.rs"]
 mod detect;
 
 #[cfg(not(portable_atomic_no_asm))]
@@ -72,11 +72,18 @@ unsafe fn cmpxchg16b(
     success: Ordering,
     failure: Ordering,
 ) -> (u128, bool) {
-    #[inline]
     #[cfg_attr(
         any(all(test, portable_atomic_nightly), portable_atomic_cmpxchg16b_dynamic),
         target_feature(enable = "cmpxchg16b")
     )]
+    #[cfg_attr(
+        any(portable_atomic_target_feature_cmpxchg16b, target_feature = "cmpxchg16b"),
+        inline
+    )]
+    #[cfg_attr(
+        not(any(portable_atomic_target_feature_cmpxchg16b, target_feature = "cmpxchg16b")),
+        inline(never)
+    )]
     unsafe fn _cmpxchg16b(
         dst: *mut u128,
         old: u128,
@@ -84,19 +91,13 @@ unsafe fn cmpxchg16b(
         success: Ordering,
         failure: Ordering,
     ) -> (u128, bool) {
-        #[cfg(all(
-            portable_atomic_cmpxchg16b_stdsimd,
-            any(portable_atomic_target_feature_cmpxchg16b, target_feature = "cmpxchg16b", miri)
-        ))]
+        #[cfg(portable_atomic_cmpxchg16b_stdsimd)]
         // SAFETY: the caller must uphold the safety contract for `_cmpxchg16b`.
         unsafe {
             let res = core::arch::x86_64::cmpxchg16b(dst, old, new, success, failure);
             (res, res == old)
         }
-        #[cfg(not(all(
-            portable_atomic_cmpxchg16b_stdsimd,
-            any(portable_atomic_target_feature_cmpxchg16b, target_feature = "cmpxchg16b", miri)
-        )))]
+        #[cfg(not(portable_atomic_cmpxchg16b_stdsimd))]
         // SAFETY: the caller must uphold the safety contract for `_cmpxchg16b`.
         unsafe {
             let _ = (success, failure);
@@ -536,8 +537,8 @@ mod tests {
     use super::*;
     use crate::utils::AtomicRepr;
 
-    test_atomic_int!(test_atomic_i128, AtomicI128, i128);
-    test_atomic_int!(test_atomic_u128, AtomicU128, u128);
+    test_atomic_int!(i128);
+    test_atomic_int!(u128);
 
     #[test]
     #[cfg_attr(miri, ignore)] // Miri doesn't support inline assembly
@@ -555,7 +556,7 @@ mod tests {
 
         ::quickcheck::quickcheck! {
             #[cfg_attr(miri, ignore)] // Miri doesn't support inline assembly
-            #[cfg_attr(sanitize_thread, ignore)] // TSan doesn't know the semantic of the asm synchronization instructions.
+            #[cfg_attr(sanitize_thread, ignore)] // TSan doesn't know the semantics of the asm synchronization instructions.
             fn test(x: u128, y: u128, z: u128) -> bool {
                 assert!(std::is_x86_feature_detected!("cmpxchg16b"));
                 unsafe {
