@@ -23,7 +23,7 @@ use super::riscv as atomic;
 #[cfg(target_arch = "arm")]
 use core::sync::atomic;
 
-#[cfg_attr(thumbv6m, path = "thumbv6m.rs")]
+#[cfg_attr(portable_atomic_armv6m, path = "armv6m.rs")]
 #[cfg_attr(target_arch = "avr", path = "avr.rs")]
 #[cfg_attr(target_arch = "msp430", path = "msp430.rs")]
 #[cfg_attr(any(target_arch = "riscv32", target_arch = "riscv64"), path = "riscv.rs")]
@@ -75,7 +75,6 @@ unsafe impl Sync for AtomicBool {}
 
 impl AtomicBool {
     #[inline]
-    #[must_use]
     pub(crate) const fn new(v: bool) -> Self {
         Self { v: UnsafeCell::new(v as u8) }
     }
@@ -567,16 +566,16 @@ atomic_int!(load_store_critical_session, AtomicU128, u128, 16);
 mod tests {
     use super::*;
 
-    test_atomic_bool_single_thread!(test_atomic_bool, AtomicBool);
-    test_atomic_ptr_single_thread!(test_atomic_ptr, AtomicPtr<u8>);
-    test_atomic_int_single_thread!(test_atomic_i8, AtomicI8, i8);
-    test_atomic_int_single_thread!(test_atomic_u8, AtomicU8, u8);
-    test_atomic_int_single_thread!(test_atomic_i16, AtomicI16, i16);
-    test_atomic_int_single_thread!(test_atomic_u16, AtomicU16, u16);
-    test_atomic_int_single_thread!(test_atomic_i32, AtomicI32, i32);
-    test_atomic_int_single_thread!(test_atomic_u32, AtomicU32, u32);
-    test_atomic_int_single_thread!(test_atomic_i64, AtomicI64, i64);
-    test_atomic_int_single_thread!(test_atomic_u64, AtomicU64, u64);
-    test_atomic_int_single_thread!(test_atomic_isize, AtomicIsize, isize);
-    test_atomic_int_single_thread!(test_atomic_usize, AtomicUsize, usize);
+    test_atomic_bool_single_thread!();
+    test_atomic_ptr_single_thread!();
+    test_atomic_int_single_thread!(i8);
+    test_atomic_int_single_thread!(u8);
+    test_atomic_int_single_thread!(i16);
+    test_atomic_int_single_thread!(u16);
+    test_atomic_int_single_thread!(i32);
+    test_atomic_int_single_thread!(u32);
+    test_atomic_int_single_thread!(i64);
+    test_atomic_int_single_thread!(u64);
+    test_atomic_int_single_thread!(isize);
+    test_atomic_int_single_thread!(usize);
 }
