@@ -10,9 +10,9 @@ Portable atomic types including support for 128-bit atomics, atomic float, etc.
 
 ## 128-bit atomics support (AtomicI128,AtomicU128)
 
-Native 128-bit atomic operations are only available for x86_64 and aarch64 at Rust 1.59+, otherwise the fallback implementation is used.
+Native 128-bit atomic operations are available on x86_64 and aarch64 at Rust 1.59+, and on s390x at only nightly, otherwise the fallback implementation is used.
 
-On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxchg16b` target feature is not enabled at compile-time, this uses the fallback implementation. `cmpxchg16b` is enabled by default only on macOS.
+On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxchg16b` target feature is not enabled at compile-time, this uses the fallback implementation. `cmpxchg16b` target feature is enabled by default only on macOS.
 
 ## Optional features
 
@@ -27,9 +27,9 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
   This allows maintaining support for older CPUs while using features that are not supported on older CPUs, such as cmpxchg16b (x86_64) and LSE (aarch64).
 
   Note:
-  - Dynamic detection is currently only enabled in Rust 1.61+ for aarch64 and in nightly for others, otherwise it works the same as the default.
+  - Dynamic detection is currently only enabled in Rust 1.61+ for aarch64 and in nightly for other platforms, otherwise it works the same as the default.
   - If the required target features are enabled at compile-time, the atomic operations are inlined.
-  - This is compatible with no-std (as with all features except `std` and `parking_lot`).
+  - This is compatible with no-std (as with all features except `std`).
 
 - **`float`**<br>
   Provide `AtomicF{32,64}`.
@@ -49,13 +49,6 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
   Note:
   - The MSRV when this feature enables depends on the MSRV of [serde].
 
-- **`parking_lot`**<br>
-  Use [parking_lot] in global locks of fallback implementation.
-
-  Note:
-  - This is not compatible with no-std.
-  - The MSRV when this feature enables depends on the MSRV of [parking_lot].
-
 ## Optional cfg
 
 - **`--cfg portable_atomic_unsafe_assume_single_core`**<br>
@@ -66,6 +59,8 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
 
   This is intentionally not an optional feature. (If this is an optional feature, dependencies can implicitly enable the feature, resulting in the use of unsound code without the end-user being aware of it.)
 
+  Enabling this cfg for targets that have atomic CAS will result in a compile error.
+
 ## Related Projects
 
 - [atomic-maybe-uninit]: Atomic operations on potentially uninitialized integers.
@@ -73,7 +68,6 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
 
 [atomic-memcpy]: https://github.com/taiki-e/atomic-memcpy
 [atomic-maybe-uninit]: https://github.com/taiki-e/atomic-maybe-uninit
-[parking_lot]: https://github.com/Amanieu/parking_lot
 [serde]: https://github.com/serde-rs/serde
 */
 
@@ -95,11 +89,13 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
 #![cfg_attr(not(portable_atomic_no_unsafe_op_in_unsafe_fn), warn(unsafe_op_in_unsafe_fn))] // unsafe_op_in_unsafe_fn requires Rust 1.52
 #![cfg_attr(portable_atomic_no_unsafe_op_in_unsafe_fn, allow(unused_unsafe))]
 #![warn(
+    clippy::default_union_representation,
     clippy::exhaustive_enums,
     clippy::exhaustive_structs,
     clippy::inline_asm_x86_att_syntax,
     clippy::missing_inline_in_public_items,
     clippy::pedantic,
+    clippy::transmute_undefined_repr,
     clippy::undocumented_unsafe_blocks
 )]
 #![allow(
@@ -109,7 +105,6 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
     clippy::inline_always,
     clippy::missing_errors_doc,
     clippy::module_inception,
-    clippy::must_use_candidate,
     clippy::type_complexity
 )]
 // 128-bit atomic
@@ -121,6 +116,10 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
     any(all(test, portable_atomic_nightly), portable_atomic_cmpxchg16b_stdsimd),
     feature(stdsimd, cmpxchg16b_target_feature)
 )]
+#![cfg_attr(
+    any(all(test, portable_atomic_nightly), portable_atomic_s390x_atomic_128),
+    feature(core_intrinsics)
+)]
 // cfg(cfg_target_has_atomic) on old nightly
 // This feature has not been changed since the change in nightly-2019-10-14
 // until it was stabilized in nightly-2022-02-11, so it can be safely enabled in
@@ -136,7 +135,7 @@ On x86_64, when the `outline-atomics` optional feature is not enabled and `cmpxc
     all(
         portable_atomic_nightly,
         any(
-            thumbv6m,
+            portable_atomic_armv6m,
             all(target_arch = "riscv32", portable_atomic_no_atomic_load_store),
             target_arch = "aarch64",
             target_arch = "x86_64",
@@ -172,14 +171,14 @@ compile_error!(
     not(portable_atomic_cfg_target_has_atomic),
     cfg(any(
         not(portable_atomic_no_atomic_cas),
-        not(any(thumbv6m, target_arch = "riscv32", target_pointer_width = "16"))
+        not(any(portable_atomic_armv6m, target_arch = "riscv32", target_pointer_width = "16"))
     ))
 )]
 #[cfg_attr(
     portable_atomic_cfg_target_has_atomic,
     cfg(any(
         target_has_atomic = "ptr",
-        not(any(thumbv6m, target_arch = "riscv32", target_pointer_width = "16"))
+        not(any(portable_atomic_armv6m, target_arch = "riscv32", target_pointer_width = "16"))
     ))
 )]
 compile_error!(
@@ -315,6 +314,7 @@ impl AtomicBool {
     /// atomic instructions, global locks for every potentially
     /// concurrent atomic operation will be used.
     #[inline]
+    #[must_use]
     pub fn is_lock_free() -> bool {
         <imp::AtomicBool as crate::utils::AtomicRepr>::is_lock_free()
     }
@@ -328,6 +328,7 @@ impl AtomicBool {
     /// **Note:** If the atomic operation relies on dynamic CPU feature detection,
     /// this type may be lock-free even if the function returns false.
     #[inline]
+    #[must_use]
     pub const fn is_always_lock_free() -> bool {
         <imp::AtomicBool as crate::utils::AtomicRepr>::IS_ALWAYS_LOCK_FREE
     }
@@ -675,6 +676,7 @@ impl<T> std::panic::RefUnwindSafe for AtomicPtr<T> {}
 impl<T> AtomicPtr<T> {
     /// Creates a new `AtomicPtr`.
     #[inline]
+    #[must_use]
     pub const fn new(p: *mut T) -> Self {
         Self { inner: imp::AtomicPtr::new(p), _marker: PhantomData }
     }
@@ -685,6 +687,7 @@ impl<T> AtomicPtr<T> {
     /// atomic instructions, global locks for every potentially
     /// concurrent atomic operation will be used.
     #[inline]
+    #[must_use]
     pub fn is_lock_free() -> bool {
         <imp::AtomicPtr<T> as crate::utils::AtomicRepr>::is_lock_free()
     }
@@ -698,6 +701,7 @@ impl<T> AtomicPtr<T> {
     /// **Note:** If the atomic operation relies on dynamic CPU feature detection,
     /// this type may be lock-free even if the function returns false.
     #[inline]
+    #[must_use]
     pub const fn is_always_lock_free() -> bool {
         <imp::AtomicPtr<T> as crate::utils::AtomicRepr>::IS_ALWAYS_LOCK_FREE
     }
@@ -962,6 +966,7 @@ atomic instructions or locks will be used.
         impl $atomic_type {
             /// Creates a new atomic integer.
             #[inline]
+            #[must_use]
             pub const fn new(v: $int_type) -> Self {
                 Self { inner: imp::$atomic_type::new(v), _marker: PhantomData }
             }
@@ -972,6 +977,7 @@ atomic instructions or locks will be used.
             /// atomic instructions, global locks for every potentially
             /// concurrent atomic operation will be used.
             #[inline]
+            #[must_use]
             pub fn is_lock_free() -> bool {
                 <imp::$atomic_type as crate::utils::AtomicRepr>::is_lock_free()
             }
@@ -985,6 +991,7 @@ atomic instructions or locks will be used.
             /// **Note:** If the atomic operation relies on dynamic CPU feature detection,
             /// this type may be lock-free even if the function returns false.
             #[inline]
+            #[must_use]
             pub const fn is_always_lock_free() -> bool {
                 <imp::$atomic_type as crate::utils::AtomicRepr>::IS_ALWAYS_LOCK_FREE
             }
@@ -1370,17 +1377,19 @@ atomic instructions or locks will be used.
                             target_arch = "aarch64",
                             any(portable_atomic_target_feature_lse, target_feature = "lse"),
                         ),
-                        armv5te,
-                        all(target_arch = "mips", target_endian = "little"),
-                        all(target_arch = "mips64", target_endian = "little"),
+                        portable_atomic_armv5te,
+                        target_arch = "mips",
+                        target_arch = "mips64",
                         target_arch = "powerpc",
                         target_arch = "powerpc64",
                     ))]
                     {
                         // HACK: the following operations are currently broken (at least on qemu):
-                        // - aarch64+lse's `AtomicI{8,16}::fetch_{max,min}` (release mode)
+                        // - aarch64's `AtomicI{8,16}::fetch_{max,min}` (release mode + lse)
                         // - armv5te's `Atomic{I,U}{8,16}::fetch_{max,min}`
+                        // - mips's `AtomicI8::fetch_{max,min}` (release mode)
                         // - mipsel's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
+                        // - mips64's `AtomicI8::fetch_{max,min}` (release mode)
                         // - mips64el's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
                         // - powerpc's `AtomicI{8,16}::fetch_{max,min}`
                         // - powerpc64's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
@@ -1437,9 +1446,9 @@ atomic instructions or locks will be used.
                             target_arch = "aarch64",
                             any(portable_atomic_target_feature_lse, target_feature = "lse"),
                         ),
-                        armv5te,
-                        all(target_arch = "mips", target_endian = "little"),
-                        all(target_arch = "mips64", target_endian = "little"),
+                        portable_atomic_armv5te,
+                        target_arch = "mips",
+                        target_arch = "mips64",
                         target_arch = "powerpc",
                         target_arch = "powerpc64",
                     ))]
@@ -1447,7 +1456,9 @@ atomic instructions or locks will be used.
                         // HACK: the following operations are currently broken (at least on qemu):
                         // - aarch64's `AtomicI{8,16}::fetch_{max,min}` (release mode + lse)
                         // - armv5te's `Atomic{I,U}{8,16}::fetch_{max,min}`
+                        // - mips's `AtomicI8::fetch_{max,min}` (release mode)
                         // - mipsel's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
+                        // - mips64's `AtomicI8::fetch_{max,min}` (release mode)
                         // - mips64el's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
                         // - powerpc's `AtomicI{8,16}::fetch_{max,min}`
                         // - powerpc64's `AtomicI{8,16}::fetch_{max,min}` (debug mode, at least)
@@ -1534,6 +1545,7 @@ This type has the same in-memory representation as the underlying floating point
         impl $atomic_type {
             /// Creates a new atomic float.
             #[inline]
+            #[must_use]
             pub const fn new(v: $float_type) -> Self {
                 Self { v: core::cell::UnsafeCell::new(v) }
             }
@@ -1544,6 +1556,7 @@ This type has the same in-memory representation as the underlying floating point
             /// atomic instructions, global locks for every potentially
             /// concurrent atomic operation will be used.
             #[inline]
+            #[must_use]
             pub fn is_lock_free() -> bool {
                 crate::$atomic_int_type::is_lock_free()
             }
@@ -1557,6 +1570,7 @@ This type has the same in-memory representation as the underlying floating point
             /// **Note:** If the atomic operation relies on dynamic CPU feature detection,
             /// this type may be lock-free even if the function returns false.
             #[inline]
+            #[must_use]
             pub const fn is_always_lock_free() -> bool {
                 crate::$atomic_int_type::is_always_lock_free()
             }
@@ -1891,6 +1905,32 @@ This type has the same in-memory representation as the underlying floating point
                 .unwrap()
             }
 
+            /// Computes the absolute value of the current value, and sets the
+            /// new value to the result.
+            ///
+            /// Returns the previous value.
+            ///
+            /// `fetch_abs` takes an [`Ordering`] argument which describes the memory ordering
+            /// of this operation. All ordering modes are possible. Note that using
+            /// [`Acquire`] makes the store part of this operation [`Relaxed`], and
+            /// using [`Release`] makes the load part [`Relaxed`].
+            #[cfg_attr(
+                not(portable_atomic_cfg_target_has_atomic),
+                cfg(any(
+                    not(portable_atomic_no_atomic_cas),
+                    portable_atomic_unsafe_assume_single_core
+                ))
+            )]
+            #[cfg_attr(
+                portable_atomic_cfg_target_has_atomic,
+                cfg(any(target_has_atomic = "ptr", portable_atomic_unsafe_assume_single_core))
+            )]
+            #[inline]
+            pub fn fetch_abs(&self, order: Ordering) -> $float_type {
+                const ABS_MASK: $int_type = !0 / 2;
+                $float_type::from_bits(self.as_bits().fetch_and(ABS_MASK, order))
+            }
+
             // TODO: Add as_mut_ptr once it is stable on other std atomic types.
             // https://github.com/rust-lang/rust/issues/66893
 
@@ -1992,6 +2032,7 @@ atomic_int!(AtomicU64, u64, 8);
     not(feature = "fallback"),
     cfg(any(
         portable_atomic_core_atomic_128,
+        portable_atomic_s390x_atomic_128,
         all(
             not(portable_atomic_core_atomic_128),
             any(not(portable_atomic_no_asm), portable_atomic_nightly),
@@ -2022,6 +2063,7 @@ atomic_int!(AtomicI128, i128, 16);
     not(feature = "fallback"),
     cfg(any(
         portable_atomic_core_atomic_128,
+        portable_atomic_s390x_atomic_128,
         all(
             not(portable_atomic_core_atomic_128),
             any(not(portable_atomic_no_asm), portable_atomic_nightly),
